'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Test = exports.Testsuite = exports.Testsuites = undefined;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable
 class-methods-use-this
 */
/*
    This Module can be loaded via `mocha --require`.
    It is only activated if process.env.REPORT_FILE is set.
    If activated, it:
    - writes a junit-compatible xml-report to the file denoted in process.env.REPORT_FILE
      (the directory of this file is prefixed with process.env.PREFIX, if set)
    - captures all stdout/stderr from tests and redirects them to this report
    - sets mocha exit code to zero, if test results are written successful
*/

const STREAMS = ['system-out', 'system-err'];

function buildEscapes() {
  const escapes = {
    '&': '&amp;',
    "'": '&apos;',
    '"': '&quot;',
    '<': '&lt;',
    '>': '&gt;'
  };

  const regex = new RegExp(`[${Object.keys(escapes).join('')}\\x00-\\x1f]`, 'g');
  for (let i = 0; i < 32; i += 1) {
    escapes[String.fromCharCode(i)] = `&#x${i.toString(16)};`;
  }
  escapes['\t'] = '\t';
  escapes['\r'] = '\r';
  escapes['\n'] = '\n';
  return str => String(str).replace(regex, x => escapes[x]);
}

const _xmlEscape = buildEscapes();

class XMLWriter {
  _startTag(writable, name, attrs = {}) {
    writable.write(`<${name}`);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = Object.keys(attrs)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const k = _step.value;

        const v = attrs[k];
        if (v != null) {
          writable.write(` ${k}="`);
          this.escape(writable, v);
          writable.write('"');
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return writable;
  }

  openTag(writable, name, attrs) {
    return this._startTag(writable, name, attrs).write('>\n');
  }

  emptyTag(writable, name, attrs) {
    return this._startTag(writable, name, attrs).write('/>\n');
  }

  closeTag(writable, name) {
    return writable.write(`</${name}>\n`);
  }

  escape(writable, s) {
    return writable.write(_xmlEscape(s));
  }

  writeFile(output, cb) {
    // Create directory if it doesn't exist
    return (0, _mkdirp2.default)(_path2.default.dirname(output), err => {
      if (err != null) {
        return cb(err);
      }
      const writable = _fs2.default.createWriteStream(output);
      writable.write('<?xml version="1.1"?>\n');
      this.write(writable);
      return writable.end(cb);
    });
  }
}

class HasStreams extends XMLWriter {
  constructor() {
    super();
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = STREAMS[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        const s = _step2.value;

        this[s] = [];
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }
  }

  write(writable) {
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = STREAMS[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        const s = _step3.value;

        const text = this[s].join('').trim();
        if (text.length) {
          this.openTag(writable, s);
          this.escape(writable, text);
          this.closeTag(writable, s);
        }
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }
  }
}

class Testsuites extends XMLWriter {
  constructor(name) {
    super();
    this.name = name;
    this.suites = [];
  }

  write(writable) {
    this.openTag(writable, 'testsuites', { name: this.name });
    var _iteratorNormalCompletion4 = true;
    var _didIteratorError4 = false;
    var _iteratorError4 = undefined;

    try {
      for (var _iterator4 = this.suites[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
        const testsuite = _step4.value;

        testsuite.write(writable);
      }
    } catch (err) {
      _didIteratorError4 = true;
      _iteratorError4 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion4 && _iterator4.return) {
          _iterator4.return();
        }
      } finally {
        if (_didIteratorError4) {
          throw _iteratorError4;
        }
      }
    }

    return this.closeTag(writable, 'testsuites');
  }
}

class Testsuite extends HasStreams {
  constructor(name, parent) {
    super();
    this.name = name;
    this.parent = parent;
    this.tests = [];
    this.failures = 0;
    this.errors = 0;
    this.timestamp = new Date();
    this.hostname = _os2.default.hostname();
    this.properties = {};
    if (this.parent == null) {
      this.parent = new Testsuites(this.name);
    }
    this.parent.suites.push(this);
  }

  addTest(test) {
    this.tests.push(test);
  }

  end() {
    this.time = new Date() - this.timestamp;
  }

  write(writable) {
    let failures = 0;
    let passed = 0;
    const total = this.tests.length;
    var _iteratorNormalCompletion5 = true;
    var _didIteratorError5 = false;
    var _iteratorError5 = undefined;

    try {
      for (var _iterator5 = this.tests[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
        const test = _step5.value;

        if (test.failed) {
          failures += 1;
        }
        if (test.passed) {
          passed += 1;
        }
      }
    } catch (err) {
      _didIteratorError5 = true;
      _iteratorError5 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion5 && _iterator5.return) {
          _iterator5.return();
        }
      } finally {
        if (_didIteratorError5) {
          throw _iteratorError5;
        }
      }
    }

    this.openTag(writable, 'testsuite', {
      name: this.name,
      hostname: this.hostname,
      timestamp: this.timestamp.toUTCString(),
      time: this.time / 1000,
      tests: total,
      failures: this.failures + failures,
      skipped: total - failures - passed,
      errors: this.errors
    });

    const keys = Object.keys(this.properties);
    if (keys.length) {
      this.openTag(writable, 'properties');
      var _iteratorNormalCompletion6 = true;
      var _didIteratorError6 = false;
      var _iteratorError6 = undefined;

      try {
        for (var _iterator6 = keys[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
          const name = _step6.value;

          const value = this.properties[name];
          this.emptyTag(writable, 'property', { name, value });
        }
      } catch (err) {
        _didIteratorError6 = true;
        _iteratorError6 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion6 && _iterator6.return) {
            _iterator6.return();
          }
        } finally {
          if (_didIteratorError6) {
            throw _iteratorError6;
          }
        }
      }

      this.closeTag(writable, 'properties');
    }

    var _iteratorNormalCompletion7 = true;
    var _didIteratorError7 = false;
    var _iteratorError7 = undefined;

    try {
      for (var _iterator7 = this.tests[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
        const test = _step7.value;

        test.write(writable);
      }
    } catch (err) {
      _didIteratorError7 = true;
      _iteratorError7 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion7 && _iterator7.return) {
          _iterator7.return();
        }
      } finally {
        if (_didIteratorError7) {
          throw _iteratorError7;
        }
      }
    }

    super.write(writable);
    this.closeTag(writable, 'testsuite');
  }
}

function fullTitle(runnable) {
  const result = [];
  let r = runnable;
  while (r != null) {
    if (r.title !== '') {
      result.push(r.title);
    }
    r = r.parent;
  }
  return result.reverse().join('.');
}

class Test extends HasStreams {
  constructor(test, suite) {
    super();
    this.test = test;
    this.failures = [];
    this.failed = false;
    this.passed = false;
    this.suite = suite;
  }

  fail(failed, err) {
    this.failed = true;
    this.passed = false;
    let message = err && err.message ? err.message : 'unknown error';
    if (failed !== this.test) {
      message += ` (from: ${fullTitle(failed)})`;
    }
    let content = '';
    if (err && err.stack) {
      content = err.stack.replace(/^/gm, '  ');
    }
    return this.failures.push({ message, content });
  }

  pass() {
    this.passed = true;
  }

  write(writable) {
    const skipped = !(this.passed || this.failed);
    const attrs = {
      classname: `${this.suite.name}.${fullTitle(this.test.parent)}`,
      name: this.test.title
    };
    if (!skipped) {
      attrs.time = this.test.duration / 1000;
    }
    this.openTag(writable, 'testcase', attrs);
    var _iteratorNormalCompletion8 = true;
    var _didIteratorError8 = false;
    var _iteratorError8 = undefined;

    try {
      for (var _iterator8 = this.failures[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
        const failure = _step8.value;

        this.openTag(writable, 'failure', { message: failure.message });
        this.escape(writable, failure.content);
        this.closeTag(writable, 'failure');
      }
    } catch (err) {
      _didIteratorError8 = true;
      _iteratorError8 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion8 && _iterator8.return) {
          _iterator8.return();
        }
      } finally {
        if (_didIteratorError8) {
          throw _iteratorError8;
        }
      }
    }

    if (skipped) {
      this.emptyTag(writable, 'skipped');
    }
    super.write(writable);
    return this.closeTag(writable, 'testcase');
  }
}

exports.Testsuites = Testsuites;
exports.Testsuite = Testsuite;
exports.Test = Test;